#include "main.h"

#include <stdlib.h>
#include <iostream.h>
#include <GL/glut.h>
#include "control.h"
#include "lights.h"
#include "cloth.h"
#include "scene.h"
#include "camera.h"


int window_width  = 500;
int window_height = 500;


int main (int argc, char** argv)
{
  // set up the window
  glutInit(&argc, &argv[0]); 
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
  glutInitWindowPosition(100,100);
  glutInitWindowSize(window_width, window_height);
  glutCreateWindow("cloth");
 
  // initialize parameters
  init();

  // register call back functions
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);

  // wait for something to happen
  glutMainLoop();

  return 0;
}


void init()
{
  glClearColor(0.0,0.0,0.0,0.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_DEPTH_TEST);

  // main menu
  make_menu();

  // callbacks
  glutKeyboardFunc(keyboard_func);
  glutMouseFunc(mouse_click_func);
  glutMotionFunc(mouse_move_func);
  glutSpecialFunc(special_func);
  glutIdleFunc(idle_func);
  glutVisibilityFunc(visibility_func);

  initSceneDisplayLists();
  initCloth();
  make_lights();
}


void display(void) 
{
  // check if there have been any openGL problems
  GLenum errCode = glGetError();
  if (errCode != GL_NO_ERROR) 
  {
      const GLubyte* errString = gluErrorString(errCode);
      cerr << "OpenGL error: " << errString << endl;
  }

  // clear the frame buffer
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

  position_camera();

  // draw the scene
  drawEverything();

  // swap buffers
  glutSwapBuffers();  

  frames++;
}


void reshape (int width, int height) 
{
  // set window height and width
  window_width  = width;
  window_height = height; 

  // the lower left corner of the viewport is 0,0
  // the upper right corner is width, height
  glViewport(0, 0, (GLint) width, (GLint) height);  

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(frustum[0], frustum[1], frustum[2], frustum[3],
            frustum[4], frustum[5]);
  glPushMatrix();

  // default mode should be modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}
