#include "scene.h"

#include <GL/glut.h>
#include <iostream.h>
#include "cloth.h"
#include "lights.h"
#include "main.h"


int drawFloor_list;


void initSceneDisplayLists ()
{
  drawFloor_list = glGenLists(FLOOR_D);
  glNewList(drawFloor_list, GL_COMPILE);
  drawFloor();
  glEndList();
}


void drawEverything ()
{
  drawRoom();
  drawCloth();
}


void drawRoom ()
{
  if (sphere)
  {
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,
                 (float[4]){ 0, 0.1, 0.9, 1 });
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,
                 (float[4]){ 0, 0.1, 0.9, 1 });
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,
                 (float[4]){ 0, 0.2, 0.7, 1 });
    glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS,
                 (float[1]){ 0.9 });
    glPushMatrix();
      glTranslatef(0,0,4);
      glutSolidSphere(2, 15, 10);
    glPopMatrix();
  }

  glCallList(drawFloor_list);
}


bool floor_pattern_a (int x, int y)
{
  return (((x + y) % 2) == 0);
}


bool floor_pattern_b (int x, int y)
{
  return (((x + y) % 2) == 1);
}


void drawFloor ()
{
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  glMaterialfv(GL_FRONT, GL_AMBIENT,
               (float[4]){ 0.5, 0.5, 0.5, 0.2 });
  glMaterialfv(GL_FRONT, GL_DIFFUSE,
               (float[4]){ 1, 1, 1, 0.2 });
  glMaterialfv(GL_FRONT, GL_SPECULAR,
               (float[4]){ 1, 1, 1, 0.2 });
  glMaterialfv(GL_FRONT, GL_SHININESS,
               (float[1]){ 0.9 });

  drawTiledRect(-10, -10, 10, 10, 20, 20, floor_pattern_a);

  glMaterialfv(GL_FRONT, GL_AMBIENT,
               (float[4]){ 0.5, 0.5, 0.5, 0.5 });
  glMaterialfv(GL_FRONT, GL_DIFFUSE,
               (float[4]){ 0.6, 0.6, 0.6, 0.5 });
  glMaterialfv(GL_FRONT, GL_SPECULAR,
               (float[4]){ 1, 1, 1, 0.5 });
  glMaterialfv(GL_FRONT, GL_SHININESS,
               (float[1]){ 0.9 });

  drawTiledRect(-10, -10, 10, 10, 20, 20, floor_pattern_b);

  glPopMatrix();
}


void drawTiledRect (float x1, float y1, float x2, float y2,
                    int divx, int divy, bool (*pred)(int,int))
{
  float x  = x1;
  float y  = y1;
  float dx = (x2 - x1) / divx;
  float dy = (y2 - y1) / divy;

  for (int i = 0; i < divx; i++)
  {
      for (int j = 0; j < divy; j++)
      {
          if ((pred == NULL) || pred(i,j))
          {
              glBegin(GL_POLYGON);
              glNormal3f(0,0,1);
              glVertex3f(x,y,0);
              glVertex3f(x+dx,y,0);
              glVertex3f(x+dx,y+dy,0);
              glVertex3f(x,y+dy,0);
              glEnd();
          }

          y += dy;
      }
      x += dx;
      y = y1;
  }
}


